@@ -12,18 +12,18 @@ import 'package:analysis_server/src/provisional/completion/completion_core.dart'
 import 'package:analysis_server/src/provisional/completion/dart/completion_dart.dart';
 import 'package:analysis_server/src/services/completion/dart/completion_manager.dart';
 import 'package:analysis_server/src/services/completion/dart/contribution_sorter.dart';
+import 'package:test/test.dart';
 import 'package:test_reflective_loader/test_reflective_loader.dart';
-import 'package:unittest/unittest.dart';
 
 import 'analysis_abstract.dart';
 import 'domain_completion_util.dart';
 import 'mocks.dart' show pumpEventQueue;
-import 'utils.dart';
 
 main() {
-  initializeTestEnvironment();
-  defineReflectiveTests(CompletionDomainHandlerTest);
-  defineReflectiveTests(_NoSearchEngine);
+  defineReflectiveSuite(() {
+    defineReflectiveTests(CompletionDomainHandlerTest);
+    defineReflectiveTests(_NoSearchEngine);
+  });
 }
 
 @reflectiveTest
@@ -48,6 +48,15 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
     expect(suggestions, hasLength(2));
   }
 
+  test_ArgumentList_imported_function_named_param2() async {
+    addTestFile('mainx() {A a = new A(); a.foo(one: 7, ^);}'
+        'class A { foo({one, two}) {} }');
+    await getSuggestions();
+    assertHasResult(CompletionSuggestionKind.NAMED_ARGUMENT, 'two: ',
+        relevance: DART_RELEVANCE_NAMED_PARAMETER);
+    expect(suggestions, hasLength(1));
+  }
+
   test_ArgumentList_imported_function_named_param_label1() async {
     addTestFile('main() { int.parse("16", r^: 16);}');
     await getSuggestions();
@@ -68,15 +77,6 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
     expect(suggestions, hasLength(2));
   }
 
-  test_ArgumentList_imported_function_named_param2() async {
-    addTestFile('mainx() {A a = new A(); a.foo(one: 7, ^);}'
-        'class A { foo({one, two}) {} }');
-    await getSuggestions();
-    assertHasResult(CompletionSuggestionKind.NAMED_ARGUMENT, 'two: ',
-        relevance: DART_RELEVANCE_NAMED_PARAMETER);
-    expect(suggestions, hasLength(1));
-  }
-
   test_html() {
     testFile = '/project/web/test.html';
     addTestFile('''
@@ -89,6 +89,20 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
     });
   }
 
+  test_import_uri_with_trailing() {
+    addFile('/project/bin/testA.dart', 'library libA;');
+    addTestFile('''
+      import '/project/bin/t^.dart';
+      main() {}''');
+    return getSuggestions().then((_) {
+      expect(replacementOffset, equals(completionOffset - 14));
+      expect(replacementLength, equals(5 + 14));
+      assertHasResult(
+          CompletionSuggestionKind.IMPORT, '/project/bin/testA.dart');
+      assertNoResult('test');
+    });
+  }
+
   test_imports() {
     addTestFile('''
       import 'dart:html';
@@ -109,13 +123,13 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
         c^''');
 
     // Make a request for suggestions
-    Request request =
+    Request request1 =
         new CompletionGetSuggestionsParams(testFile, completionOffset)
             .toRequest('7');
-    Response response = handleSuccessfulRequest(request);
-    var result1 = new CompletionGetSuggestionsResult.fromResponse(response);
+    Response response1 = await waitResponse(request1);
+    var result1 = new CompletionGetSuggestionsResult.fromResponse(response1);
     var completionId1 = result1.id;
-    assertValidId(response.id);
+    assertValidId(completionId1);
 
     // Perform some analysis but assert that no suggestions have yet been made
     completionId = completionId1;
@@ -127,7 +141,7 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
     Request request2 =
         new CompletionGetSuggestionsParams(testFile, completionOffset)
             .toRequest('8');
-    Response response2 = handleSuccessfulRequest(request2);
+    Response response2 = await waitResponse(request2);
     var result2 = new CompletionGetSuggestionsResult.fromResponse(response2);
     var completionId2 = result2.id;
     assertValidId(completionId2);
@@ -150,7 +164,7 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
     Request request =
         new CompletionGetSuggestionsParams(testFile, completionOffset)
             .toRequest('0');
-    Response response = handleSuccessfulRequest(request);
+    Response response = await waitResponse(request);
     completionId = response.id;
     assertValidId(completionId);
 
@@ -213,7 +227,7 @@ class CompletionDomainHandlerTest extends AbstractCompletionDomainTest {
         {testFile: new AddContentOverlay(revisedContent)}).toRequest('add1'));
 
     // Request code completion immediately after edit
-    Response response = handleSuccessfulRequest(
+    Response response = await waitResponse(
         new CompletionGetSuggestionsParams(testFile, completionOffset)
             .toRequest('0'));
     completionId = response.id;
@@ -452,12 +466,12 @@ class B extends A {
     });
   }
 
-  test_offset_past_eof() {
+  test_offset_past_eof() async {
     addTestFile('main() { }', offset: 300);
     Request request =
         new CompletionGetSuggestionsParams(testFile, completionOffset)
             .toRequest('0');
-    Response response = handler.handleRequest(request);
+    Response response = await waitResponse(request);
     expect(response.id, '0');
     expect(response.error.code, RequestErrorCode.INVALID_PARAMETER);
   }
@@ -561,20 +575,6 @@ class B extends A {m() {^}}
       assertNoResult('HtmlElement');
     });
   }
-
-  test_import_uri_with_trailing() {
-    addFile('/project/bin/testA.dart', 'library libA;');
-    addTestFile('''
-      import '/project/bin/t^.dart';
-      main() {}''');
-    return getSuggestions().then((_) {
-      expect(replacementOffset, equals(completionOffset - 14));
-      expect(replacementLength, equals(5 + 14));
-      assertHasResult(
-          CompletionSuggestionKind.IMPORT, '/project/bin/testA.dart');
-      assertNoResult('test');
-    });
-  }
 }
 
 class MockRelevancySorter implements DartContributionSorter {
