@@ -6,6 +6,7 @@ library test.integration.completion.all;
 
 import 'package:test_reflective_loader/test_reflective_loader.dart';
 
+import 'get_suggestions_driver_test.dart' as get_suggestions_driver_test;
 import 'get_suggestions_test.dart' as get_suggestions_test;
 
 /**
@@ -13,6 +14,7 @@ import 'get_suggestions_test.dart' as get_suggestions_test;
  */
 main() {
   defineReflectiveSuite(() {
+    get_suggestions_driver_test.main();
     get_suggestions_test.main();
   }, name: 'completion');
 }
