@@ -9,10 +9,13 @@ import 'package:analyzer/dart/ast/visitor.dart';
 import 'package:analyzer/dart/element/element.dart';
 import 'package:analyzer/dart/element/visitor.dart';
 import 'package:analyzer/src/dart/analysis/driver.dart';
+import 'package:analyzer/src/dart/analysis/file_state.dart';
 import 'package:analyzer/src/dart/analysis/index.dart';
 import 'package:analyzer/src/dart/ast/utilities.dart';
 import 'package:analyzer/src/dart/element/element.dart';
 import 'package:analyzer/src/dart/element/member.dart';
+import 'package:analyzer/src/dart/resolver/scope.dart' show NamespaceBuilder;
+import 'package:analyzer/src/generated/utilities_dart.dart';
 import 'package:analyzer/src/summary/idl.dart';
 import 'package:collection/collection.dart';
 
@@ -30,6 +33,30 @@ class Search {
 
   Search(this._driver);
 
+  /**
+   * Returns class members with names matching the given [regExp].
+   */
+  Future<List<Element>> classMembers(RegExp regExp) async {
+    List<Element> elements = <Element>[];
+
+    void addElement(Element element) {
+      if (!element.isSynthetic && regExp.hasMatch(element.displayName)) {
+        elements.add(element);
+      }
+    }
+
+    for (FileState file in _driver.fsState.knownFiles) {
+      CompilationUnitElement unitElement =
+          await _driver.getUnitElement(file.path);
+      for (ClassElement clazz in unitElement.types) {
+        clazz.accessors.forEach(addElement);
+        clazz.fields.forEach(addElement);
+        clazz.methods.forEach(addElement);
+      }
+    }
+    return elements;
+  }
+
   /**
    * Returns references to the [element].
    */
@@ -40,11 +67,12 @@ class Search {
 
     ElementKind kind = element.kind;
     if (kind == ElementKind.CLASS ||
-        kind == ElementKind.COMPILATION_UNIT ||
         kind == ElementKind.CONSTRUCTOR ||
         kind == ElementKind.FUNCTION_TYPE_ALIAS ||
         kind == ElementKind.SETTER) {
       return _searchReferences(element);
+    } else if (kind == ElementKind.COMPILATION_UNIT) {
+      return _searchReferences_CompilationUnit(element);
     } else if (kind == ElementKind.GETTER) {
       return _searchReferences_Getter(element);
     } else if (kind == ElementKind.FIELD ||
@@ -55,9 +83,13 @@ class Search {
         return _searchReferences_Local(element, (n) => n is Block);
       }
       return _searchReferences_Function(element);
+    } else if (kind == ElementKind.IMPORT) {
+      return _searchReferences_Import(element);
     } else if (kind == ElementKind.LABEL ||
         kind == ElementKind.LOCAL_VARIABLE) {
       return _searchReferences_Local(element, (n) => n is Block);
+    } else if (kind == ElementKind.LIBRARY) {
+      return _searchReferences_Library(element);
     } else if (kind == ElementKind.PARAMETER) {
       return _searchReferences_Parameter(element);
     } else if (kind == ElementKind.PREFIX) {
@@ -66,46 +98,165 @@ class Search {
       return _searchReferences_Local(
           element, (n) => n.parent is CompilationUnit);
     }
-    // TODO(scheglov) support other kinds
     return const <SearchResult>[];
   }
 
+  /**
+   * Returns subtypes of the given [type].
+   */
+  Future<List<SearchResult>> subTypes(ClassElement type) async {
+    if (type == null) {
+      return const <SearchResult>[];
+    }
+    List<SearchResult> results = <SearchResult>[];
+    await _addResults(results, type, const {
+      IndexRelationKind.IS_EXTENDED_BY: SearchResultKind.REFERENCE,
+      IndexRelationKind.IS_MIXED_IN_BY: SearchResultKind.REFERENCE,
+      IndexRelationKind.IS_IMPLEMENTED_BY: SearchResultKind.REFERENCE
+    });
+    return results;
+  }
+
+  /**
+   * Returns top-level elements with names matching the given [regExp].
+   */
+  Future<List<Element>> topLevelElements(RegExp regExp) async {
+    List<Element> elements = <Element>[];
+
+    void addElement(Element element) {
+      if (!element.isSynthetic && regExp.hasMatch(element.displayName)) {
+        elements.add(element);
+      }
+    }
+
+    for (FileState file in _driver.fsState.knownFiles) {
+      CompilationUnitElement unitElement =
+          await _driver.getUnitElement(file.path);
+      unitElement.accessors.forEach(addElement);
+      unitElement.enums.forEach(addElement);
+      unitElement.functions.forEach(addElement);
+      unitElement.functionTypeAliases.forEach(addElement);
+      unitElement.topLevelVariables.forEach(addElement);
+      unitElement.types.forEach(addElement);
+    }
+    return elements;
+  }
+
+  /**
+   * Returns unresolved references to the given [name].
+   */
+  Future<List<SearchResult>> unresolvedMemberReferences(String name) async {
+    if (name == null) {
+      return const <SearchResult>[];
+    }
+
+    // Prepare the list of files that reference the name.
+    List<String> files = await _driver.getFilesReferencingName(name);
+
+    // Check the index of every file that references the element name.
+    List<SearchResult> results = [];
+    for (String file in files) {
+      AnalysisDriverUnitIndex index = await _driver.getIndex(file);
+      _IndexRequest request = new _IndexRequest(index);
+      var fileResults = await request.getUnresolvedMemberReferences(
+          name,
+          const {
+            IndexRelationKind.IS_READ_BY: SearchResultKind.READ,
+            IndexRelationKind.IS_WRITTEN_BY: SearchResultKind.WRITE,
+            IndexRelationKind.IS_READ_WRITTEN_BY: SearchResultKind.READ_WRITE,
+            IndexRelationKind.IS_INVOKED_BY: SearchResultKind.INVOCATION
+          },
+          () => _driver.getUnitElement(file));
+      results.addAll(fileResults);
+    }
+
+    return results;
+  }
+
   Future<Null> _addResults(List<SearchResult> results, Element element,
       Map<IndexRelationKind, SearchResultKind> relationToResultKind) async {
-    String path = element.source.fullName;
-
-    // If the file with the element is not known, then the element is not used.
-    if (!_driver.knownFiles.contains(path)) {
-      return;
+    // Prepare the element name.
+    String name = element.displayName;
+    if (element is ConstructorElement) {
+      name = element.enclosingElement.displayName;
     }
 
-    // TODO(scheglov) optimize for private elements
-    String name = element.displayName;
-
     // Prepare the list of files that reference the element name.
-    List<String> files = await _driver.getFilesReferencingName(name);
-    if (!files.contains(path) && _driver.addedFiles.contains(path)) {
-      files.add(path);
+    List<String> files = <String>[];
+    String path = element.source.fullName;
+    if (name.startsWith('_')) {
+      String libraryPath = element.library.source.fullName;
+      if (_driver.addedFiles.contains(libraryPath)) {
+        FileState library = _driver.fsState.getFileForPath(libraryPath);
+        List<FileState> candidates = [library]..addAll(library.partedFiles);
+        for (FileState file in candidates) {
+          if (file.path == path || file.referencedNames.contains(name)) {
+            files.add(file.path);
+          }
+        }
+      }
+    } else {
+      files = await _driver.getFilesReferencingName(name);
+      if (!files.contains(path) && _driver.addedFiles.contains(path)) {
+        files.add(path);
+      }
     }
 
     // Check the index of every file that references the element name.
     for (String file in files) {
-      IndexResult result = await _driver.getIndex(file);
-      _IndexRequest request = new _IndexRequest(result.index);
-      int elementId = request.findElementId(element);
-      if (elementId != -1) {
-        CompilationUnitElement unitElement = result.unitElement;
-        List<SearchResult> fileResults =
-            request.getRelations(elementId, relationToResultKind, unitElement);
-        results.addAll(fileResults);
-      }
+      await _addResultsInFile(results, element, relationToResultKind, file);
+    }
+  }
+
+  /**
+   * Add results for [element] usage in the given [file].
+   */
+  Future<Null> _addResultsInFile(
+      List<SearchResult> results,
+      Element element,
+      Map<IndexRelationKind, SearchResultKind> relationToResultKind,
+      String file) async {
+    AnalysisDriverUnitIndex index = await _driver.getIndex(file);
+    _IndexRequest request = new _IndexRequest(index);
+    int elementId = request.findElementId(element);
+    if (elementId != -1) {
+      List<SearchResult> fileResults = await request.getRelations(
+          elementId, relationToResultKind, () => _driver.getUnitElement(file));
+      results.addAll(fileResults);
     }
   }
 
   Future<List<SearchResult>> _searchReferences(Element element) async {
     List<SearchResult> results = <SearchResult>[];
     await _addResults(results, element,
-        {IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.REFERENCE});
+        const {IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.REFERENCE});
+    return results;
+  }
+
+  Future<List<SearchResult>> _searchReferences_CompilationUnit(
+      CompilationUnitElement element) async {
+    String path = element.source.fullName;
+
+    // If the path is not known, then the file is not referenced.
+    if (!_driver.fsState.knownFilePaths.contains(path)) {
+      return const <SearchResult>[];
+    }
+
+    // Check every file that references the given path.
+    List<SearchResult> results = <SearchResult>[];
+    for (FileState file in _driver.fsState.knownFiles) {
+      for (FileState referencedFile in file.directReferencedFiles) {
+        if (referencedFile.path == path) {
+          await _addResultsInFile(
+              results,
+              element,
+              const {
+                IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.REFERENCE
+              },
+              file.path);
+        }
+      }
+    }
     return results;
   }
 
@@ -115,20 +266,20 @@ class Search {
     PropertyAccessorElement getter = field.getter;
     PropertyAccessorElement setter = field.setter;
     if (!field.isSynthetic) {
-      await _addResults(results, field, {
+      await _addResults(results, field, const {
         IndexRelationKind.IS_WRITTEN_BY: SearchResultKind.WRITE,
         IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.REFERENCE
       });
     }
     if (getter != null) {
-      await _addResults(results, getter, {
+      await _addResults(results, getter, const {
         IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.READ,
         IndexRelationKind.IS_INVOKED_BY: SearchResultKind.INVOCATION
       });
     }
     if (setter != null) {
       await _addResults(results, setter,
-          {IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.WRITE});
+          const {IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.WRITE});
     }
     return results;
   }
@@ -138,7 +289,7 @@ class Search {
       element = (element as Member).baseElement;
     }
     List<SearchResult> results = <SearchResult>[];
-    await _addResults(results, element, {
+    await _addResults(results, element, const {
       IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.REFERENCE,
       IndexRelationKind.IS_INVOKED_BY: SearchResultKind.INVOCATION
     });
@@ -148,13 +299,62 @@ class Search {
   Future<List<SearchResult>> _searchReferences_Getter(
       PropertyAccessorElement getter) async {
     List<SearchResult> results = <SearchResult>[];
-    await _addResults(results, getter, {
+    await _addResults(results, getter, const {
       IndexRelationKind.IS_REFERENCED_BY: SearchResultKind.REFERENCE,
       IndexRelationKind.IS_INVOKED_BY: SearchResultKind.INVOCATION
     });
     return results;
   }
 
+  Future<List<SearchResult>> _searchReferences_Import(
+      ImportElement element) async {
+    // Search only in drivers to which the library was added.
+    String path = element.source.fullName;
+    if (!_driver.addedFiles.contains(path)) {
+      return const <SearchResult>[];
+    }
+
+    List<SearchResult> results = <SearchResult>[];
+    LibraryElement libraryElement = element.library;
+    for (CompilationUnitElement unitElement in libraryElement.units) {
+      String unitPath = unitElement.source.fullName;
+      AnalysisResult unitAnalysisResult = await _driver.getResult(unitPath);
+      _ImportElementReferencesVisitor visitor =
+          new _ImportElementReferencesVisitor(element, unitElement);
+      unitAnalysisResult.unit.accept(visitor);
+      results.addAll(visitor.results);
+    }
+    return results;
+  }
+
+  Future<List<SearchResult>> _searchReferences_Library(
+      LibraryElement element) async {
+    // Search only in drivers to which the library with the prefix was added.
+    String path = element.source.fullName;
+    if (!_driver.addedFiles.contains(path)) {
+      return const <SearchResult>[];
+    }
+
+    List<SearchResult> results = <SearchResult>[];
+    for (CompilationUnitElement unitElement in element.units) {
+      String unitPath = unitElement.source.fullName;
+      AnalysisResult unitAnalysisResult = await _driver.getResult(unitPath);
+      CompilationUnit unit = unitAnalysisResult.unit;
+      for (Directive directive in unit.directives) {
+        if (directive is PartOfDirective && directive.element == element) {
+          results.add(new SearchResult._(
+              unit.element,
+              SearchResultKind.REFERENCE,
+              directive.libraryName.offset,
+              directive.libraryName.length,
+              true,
+              false));
+        }
+      }
+    }
+    return results;
+  }
+
   Future<List<SearchResult>> _searchReferences_Local(
       Element element, bool isRootNode(AstNode n)) async {
     String path = element.source.fullName;
@@ -191,11 +391,13 @@ class Search {
   Future<List<SearchResult>> _searchReferences_Parameter(
       ParameterElement parameter) async {
     List<SearchResult> results = <SearchResult>[];
-    results.addAll(await _searchReferences(parameter));
     results.addAll(await _searchReferences_Local(parameter, (AstNode node) {
       AstNode parent = node.parent;
       return parent is ClassDeclaration || parent is CompilationUnit;
     }));
+    if (parameter.parameterKind == ParameterKind.NAMED) {
+      results.addAll(await _searchReferences(parameter));
+    }
     return results;
   }
 
@@ -225,11 +427,6 @@ class Search {
  * A single search result.
  */
 class SearchResult {
-  /**
-   * The element that is used at this result.
-   */
-  final Element element;
-
   /**
    * The deep most element that contains this result.
    */
@@ -260,8 +457,8 @@ class SearchResult {
    */
   final bool isQualified;
 
-  SearchResult._(this.element, this.enclosingElement, this.kind, this.offset,
-      this.length, this.isResolved, this.isQualified);
+  SearchResult._(this.enclosingElement, this.kind, this.offset, this.length,
+      this.isResolved, this.isQualified);
 
   @override
   String toString() {
@@ -309,6 +506,72 @@ class _ContainingElementFinder extends GeneralizingElementVisitor {
   }
 }
 
+/**
+ * Visitor that adds [SearchResult]s for references to the [importElement].
+ */
+class _ImportElementReferencesVisitor extends RecursiveAstVisitor {
+  final List<SearchResult> results = <SearchResult>[];
+
+  final ImportElement importElement;
+  final CompilationUnitElement enclosingUnitElement;
+
+  Set<Element> importedElements;
+
+  _ImportElementReferencesVisitor(
+      ImportElement element, this.enclosingUnitElement)
+      : importElement = element {
+    importedElements = new NamespaceBuilder()
+        .createImportNamespaceForDirective(element)
+        .definedNames
+        .values
+        .toSet();
+  }
+
+  @override
+  visitExportDirective(ExportDirective node) {}
+
+  @override
+  visitImportDirective(ImportDirective node) {}
+
+  @override
+  visitSimpleIdentifier(SimpleIdentifier node) {
+    if (node.inDeclarationContext()) {
+      return;
+    }
+    if (importElement.prefix != null) {
+      if (node.staticElement == importElement.prefix) {
+        AstNode parent = node.parent;
+        if (parent is PrefixedIdentifier && parent.prefix == node) {
+          if (importedElements.contains(parent.staticElement)) {
+            _addResultForPrefix(node, parent.identifier);
+          }
+        }
+        if (parent is MethodInvocation && parent.target == node) {
+          if (importedElements.contains(parent.methodName.staticElement)) {
+            _addResultForPrefix(node, parent.methodName);
+          }
+        }
+      }
+    } else {
+      if (importedElements.contains(node.staticElement)) {
+        _addResult(node.offset, 0);
+      }
+    }
+  }
+
+  void _addResult(int offset, int length) {
+    Element enclosingElement =
+        _getEnclosingElement(enclosingUnitElement, offset);
+    results.add(new SearchResult._(enclosingElement, SearchResultKind.REFERENCE,
+        offset, length, true, false));
+  }
+
+  void _addResultForPrefix(SimpleIdentifier prefixNode, AstNode nextNode) {
+    int prefixOffset = prefixNode.offset;
+    _addResult(prefixOffset, nextNode.offset - prefixOffset);
+  }
+}
+
 class _IndexRequest {
   final AnalysisDriverUnitIndex index;
 
@@ -401,11 +664,15 @@ class _IndexRequest {
   /**
    * Return a list of results where an element with the given [elementId] has
    * a relation with the kind from [relationToResultKind].
+   *
+   * The function [getEnclosingUnitElement] is used to lazily compute the
+   * enclosing [CompilationUnitElement] if there is a relation of an
+   * interesting kind.
    */
-  List<SearchResult> getRelations(
+  Future<List<SearchResult>> getRelations(
       int elementId,
       Map<IndexRelationKind, SearchResultKind> relationToResultKind,
-      CompilationUnitElement enclosingUnitElement) {
+      Future<CompilationUnitElement> getEnclosingUnitElement()) async {
     // Find the first usage of the element.
     int i = _findFirstOccurrence(index.usedElements, elementId);
     if (i == -1) {
@@ -413,6 +680,7 @@ class _IndexRequest {
     }
     // Create locations for every usage of the element.
     List<SearchResult> results = <SearchResult>[];
+    CompilationUnitElement enclosingUnitElement = null;
     for (;
         i < index.usedElements.length && index.usedElements[i] == elementId;
         i++) {
@@ -420,10 +688,10 @@ class _IndexRequest {
       SearchResultKind resultKind = relationToResultKind[relationKind];
       if (resultKind != null) {
         int offset = index.usedElementOffsets[i];
+        enclosingUnitElement ??= await getEnclosingUnitElement();
         Element enclosingElement =
             _getEnclosingElement(enclosingUnitElement, offset);
         results.add(new SearchResult._(
-            null,
             enclosingElement,
             resultKind,
             offset,
@@ -466,6 +734,45 @@ class _IndexRequest {
     return -1;
   }
 
+  /**
+   * Return a list of results where a class members with the given [name] is
+   * referenced with a qualifier, but is not resolved.
+   */
+  Future<List<SearchResult>> getUnresolvedMemberReferences(
+      String name,
+      Map<IndexRelationKind, SearchResultKind> relationToResultKind,
+      Future<CompilationUnitElement> getEnclosingUnitElement()) async {
+    // Find the name identifier.
+    int nameId = getStringId(name);
+    if (nameId == -1) {
+      return const <SearchResult>[];
+    }
+
+    // Find the first usage of the name.
+    int i = _findFirstOccurrence(index.usedNames, nameId);
+    if (i == -1) {
+      return const <SearchResult>[];
+    }
+
+    // Create results for every usage of the name.
+    List<SearchResult> results = <SearchResult>[];
+    CompilationUnitElement enclosingUnitElement = null;
+    for (; i < index.usedNames.length && index.usedNames[i] == nameId; i++) {
+      IndexRelationKind relationKind = index.usedNameKinds[i];
+      SearchResultKind resultKind = relationToResultKind[relationKind];
+      if (resultKind != null) {
+        int offset = index.usedNameOffsets[i];
+        enclosingUnitElement ??= await getEnclosingUnitElement();
+        Element enclosingElement =
+            _getEnclosingElement(enclosingUnitElement, offset);
+        results.add(new SearchResult._(enclosingElement, resultKind, offset,
+            name.length, false, index.usedNameIsQualifiedFlags[i]));
+      }
+    }
+
+    return results;
+  }
+
   /**
    * Return the identifier of the [uri] in the [index] or `-1` if the [uri] is
    * not used in the [index].
@@ -541,7 +848,7 @@ class _LocalReferencesVisitor extends RecursiveAstVisitor {
     bool isQualified = node.parent is Label;
     Element enclosingElement =
         _getEnclosingElement(enclosingUnitElement, node.offset);
-    results.add(new SearchResult._(element, enclosingElement, kind, node.offset,
-        node.length, true, isQualified));
+    results.add(new SearchResult._(
+        enclosingElement, kind, node.offset, node.length, true, isQualified));
   }
 }
