@@ -340,10 +340,6 @@ abstract class ElementHandle implements Element {
   @override
   String get displayName => actualElement.displayName;
 
-  @deprecated
-  @override
-  SourceRange get docRange => actualElement.docRange;
-
   @override
   String get documentationComment => actualElement.documentationComment;
 
@@ -413,7 +409,8 @@ abstract class ElementHandle implements Element {
       object is Element && object.location == _location;
 
   @override
-  accept(ElementVisitor visitor) => actualElement.accept(visitor);
+  /*=T*/ accept/*<T>*/(ElementVisitor<dynamic/*=T*/ > visitor) =>
+      actualElement.accept(visitor);
 
   @override
   String computeDocumentationComment() => documentationComment;
@@ -588,6 +585,9 @@ class FieldElementHandle extends PropertyInducingElementHandle
   @override
   bool get isEnumConstant => actualElement.isEnumConstant;
 
+  @override
+  bool get isVirtual => actualElement.isVirtual;
+
   @override
   ElementKind get kind => ElementKind.FIELD;
 
@@ -811,18 +811,12 @@ class LibraryElementHandle extends ElementHandle implements LibraryElement {
   @override
   List<CompilationUnitElement> get units => actualElement.units;
 
-  @override
-  List<LibraryElement> get visibleLibraries => actualElement.visibleLibraries;
-
   @override
   List<ImportElement> getImportsWithPrefix(PrefixElement prefixElement) =>
       actualElement.getImportsWithPrefix(prefixElement);
 
   @override
   ClassElement getType(String className) => actualElement.getType(className);
-
-  @override
-  bool isUpToDate(int timeStamp) => actualElement.isUpToDate(timeStamp);
 }
 
 /**
@@ -881,6 +875,10 @@ class MethodElementHandle extends ExecutableElementHandle
 
   @override
   MethodDeclaration computeNode() => actualElement.computeNode();
+
+  @override
+  FunctionType getReifiedType(DartType objectType) =>
+      actualElement.getReifiedType(objectType);
 }
 
 /**
@@ -904,6 +902,9 @@ class ParameterElementHandle extends VariableElementHandle
   @override
   String get defaultValueCode => actualElement.defaultValueCode;
 
+  @override
+  bool get isCovariant => actualElement.isCovariant;
+
   @override
   bool get isInitializingFormal => actualElement.isInitializingFormal;
 
