@@ -4,11 +4,9 @@
 
 library analyzer.test.source.test_all;
 
-import 'package:unittest/unittest.dart';
+import 'package:test_reflective_loader/test_reflective_loader.dart';
 
-import '../utils.dart';
 import 'analysis_options_provider_test.dart' as analysis_options_provider_test;
-import 'config_test.dart' as config_test;
 import 'embedder_test.dart' as embedder_test;
 import 'error_processor_test.dart' as error_processor_test;
 import 'package_map_provider_test.dart' as package_map_provider_test;
@@ -18,15 +16,13 @@ import 'sdk_ext_test.dart' as sdk_ext_test;
 
 /// Utility for manually running all tests.
 main() {
-  initializeTestEnvironment();
-  group('source', () {
+  defineReflectiveSuite(() {
     analysis_options_provider_test.main();
-    config_test.main();
     embedder_test.main();
     error_processor_test.main();
     package_map_provider_test.main();
     package_map_resolver_test.main();
     path_filter_test.main();
     sdk_ext_test.main();
-  });
+  }, name: 'source');
 }
