@@ -5,6 +5,7 @@
 import 'dart:async';
 
 import 'package:analyzer/dart/ast/ast.dart';
+import 'package:analyzer/dart/ast/standard_resolution_map.dart';
 import 'package:analyzer/dart/element/element.dart';
 import 'package:analyzer/src/dart/analysis/driver.dart';
 import 'package:analyzer/src/dart/analysis/search.dart';
@@ -70,6 +71,144 @@ class SearchTest extends BaseAnalysisDriverTest {
   CompilationUnitElement testUnitElement;
   LibraryElement testLibraryElement;
 
+  test_classMembers() async {
+    await _resolveTestUnit('''
+class A {
+  test() {}
+}
+class B {
+  int test = 1;
+  int testTwo = 2;
+  main() {
+    int test = 3;
+  }
+}
+''');
+    ClassElement a = _findElement('A');
+    ClassElement b = _findElement('B');
+    RegExp regExp = new RegExp(r'^test$');
+    expect(await driver.search.classMembers(regExp),
+        unorderedEquals([a.methods[0], b.fields[0]]));
+  }
+
+  test_classMembers_importNotDart() async {
+    await _resolveTestUnit('''
+import 'not-dart.txt';
+''');
+    RegExp regExp = new RegExp(r'^test$');
+    expect(await driver.search.classMembers(regExp), isEmpty);
+  }
+
+  test_searchMemberReferences_qualified_resolved() async {
+    await _resolveTestUnit('''
+class C {
+  var test;
+}
+main(C c) {
+  print(c.test);
+  c.test = 1;
+  c.test += 2;
+  c.test();
+}
+''');
+    await _verifyNameReferences('test', []);
+  }
+
+  test_searchMemberReferences_qualified_unresolved() async {
+    await _resolveTestUnit('''
+main(p) {
+  print(p.test);
+  p.test = 1;
+  p.test += 2;
+  p.test();
+}
+''');
+    Element main = _findElement('main');
+    await _verifyNameReferences('test', <ExpectedResult>[
+      _expectIdQU(main, SearchResultKind.READ, 'test);'),
+      _expectIdQU(main, SearchResultKind.WRITE, 'test = 1;'),
+      _expectIdQU(main, SearchResultKind.READ_WRITE, 'test += 2;'),
+      _expectIdQU(main, SearchResultKind.INVOCATION, 'test();'),
+    ]);
+  }
+
+  test_searchMemberReferences_unqualified_resolved() async {
+    await _resolveTestUnit('''
+class C {
+  var test;
+  main() {
+    print(test);
+    test = 1;
+    test += 2;
+    test();
+  }
+}
+''');
+    await _verifyNameReferences('test', []);
+  }
+
+  test_searchMemberReferences_unqualified_unresolved() async {
+    await _resolveTestUnit('''
+class C {
+  main() {
+    print(test);
+    test = 1;
+    test += 2;
+    test();
+  }
+}
+''');
+    Element main = _findElement('main');
+    await _verifyNameReferences('test', <ExpectedResult>[
+      _expectIdU(main, SearchResultKind.READ, 'test);'),
+      _expectIdU(main, SearchResultKind.WRITE, 'test = 1;'),
+      _expectIdU(main, SearchResultKind.READ_WRITE, 'test += 2;'),
+      _expectIdU(main, SearchResultKind.INVOCATION, 'test();'),
+    ]);
+  }
+
+  test_searchReferences_ClassElement_definedInSdk_declarationSite() async {
+    await _resolveTestUnit('''
+import 'dart:math';
+Random v1;
+Random v2;
+''');
+
+    // Find the Random class element in the SDK source.
+    // IDEA performs search always at declaration, never at reference.
+    ClassElement randomElement;
+    {
+      String randomPath = sdk.mapDartUri('dart:math').fullName;
+      AnalysisResult result = await driver.getResult(randomPath);
+      randomElement = result.unit.element.getType('Random');
+    }
+
+    Element v1 = _findElement('v1');
+    Element v2 = _findElement('v2');
+    var expected = [
+      _expectId(v1, SearchResultKind.REFERENCE, 'Random v1;'),
+      _expectId(v2, SearchResultKind.REFERENCE, 'Random v2;'),
+    ];
+    await _verifyReferences(randomElement, expected);
+  }
+
+  test_searchReferences_ClassElement_definedInSdk_useSite() async {
+    await _resolveTestUnit('''
+import 'dart:math';
+Random v1;
+Random v2;
+''');
+
+    var v1 = _findElement('v1') as VariableElement;
+    var v2 = _findElement('v2') as VariableElement;
+    var randomElement = v1.type.element as ClassElement;
+    var expected = [
+      _expectId(v1, SearchResultKind.REFERENCE, 'Random v1;'),
+      _expectId(v2, SearchResultKind.REFERENCE, 'Random v2;'),
+    ];
+    await _verifyReferences(randomElement, expected);
+  }
+
   test_searchReferences_ClassElement_definedInside() async {
     await _resolveTestUnit('''
 class A {};
@@ -121,27 +260,73 @@ main(A p) {
     await _verifyReferences(element, expected);
   }
 
-  @failingTest
   test_searchReferences_CompilationUnitElement() async {
-    provider.newFile(
-        _p('$testProject/my_part.dart'),
-        '''
-part of lib;
-''');
+    provider.newFile(_p('$testProject/foo.dart'), '');
     await _resolveTestUnit('''
-library lib;
-part 'my_part.dart';
+import 'foo.dart'; // import
+export 'foo.dart'; // export
 ''');
-    CompilationUnitElement element = _findElementAtString('my_part');
+    CompilationUnitElement element =
+        testLibraryElement.imports[0].importedLibrary.definingCompilationUnit;
+    int uriLength = "'foo.dart'".length;
     var expected = [
-      _expectIdQ(element.library.definingCompilationUnit,
-          SearchResultKind.REFERENCE, "'my_part.dart'",
-          length: "'my_part.dart'".length)
+      _expectIdQ(
+          testUnitElement, SearchResultKind.REFERENCE, "'foo.dart'; // import",
+          length: uriLength),
+      _expectIdQ(
+          testUnitElement, SearchResultKind.REFERENCE, "'foo.dart'; // export",
+          length: uriLength),
     ];
     await _verifyReferences(element, expected);
   }
 
-  test_searchReferences_ConstructorElement() async {
+  test_searchReferences_ConstructorElement_default() async {
+    await _resolveTestUnit('''
+class A {
+  A() {}
+}
+main() {
+  new A();
+}
+''');
+    ConstructorElement element = _findElementAtString('A() {}');
+    Element mainElement = _findElement('main');
+    var expected = [
+      _expectIdQ(mainElement, SearchResultKind.REFERENCE, '();', length: 0)
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ConstructorElement_default_otherFile() async {
+    String other = _p('$testProject/other.dart');
+    String otherCode = '''
+import 'test.dart';
+main() {
+  new A(); // in other
+}
+''';
+    provider.newFile(other, otherCode);
+    driver.addFile(other);
+
+    await _resolveTestUnit('''
+class A {
+  A() {}
+}
+''');
+    ConstructorElement element = _findElementAtString('A() {}');
+
+    CompilationUnit otherUnit = (await driver.getResult(other)).unit;
+    Element main =
+        resolutionMap.elementDeclaredByCompilationUnit(otherUnit).functions[0];
+    var expected = [
+      new ExpectedResult(main, SearchResultKind.REFERENCE,
+          otherCode.indexOf('(); // in other'), 0,
+          isResolved: true, isQualified: true)
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ConstructorElement_named() async {
     await _resolveTestUnit('''
 class A {
   A.named() {}
@@ -154,7 +339,7 @@ main() {
     Element mainElement = _findElement('main');
     var expected = [
       _expectIdQ(mainElement, SearchResultKind.REFERENCE, '.named();',
-          length: 6)
+          length: '.named'.length)
     ];
     await _verifyReferences(element, expected);
   }
@@ -300,6 +485,83 @@ main() {
     await _verifyReferences(element, expected);
   }
 
+  test_searchReferences_ImportElement_noPrefix() async {
+    await _resolveTestUnit('''
+import 'dart:math' show max, PI, Random hide min;
+export 'dart:math' show max, PI, Random hide min;
+main() {
+  print(PI);
+  print(new Random());
+  print(max(1, 2));
+}
+Random bar() => null;
+''');
+    ImportElement element = testLibraryElement.imports[0];
+    Element mainElement = await _findElement('main');
+    Element barElement = await _findElement('bar');
+    var kind = SearchResultKind.REFERENCE;
+    var expected = [
+      _expectId(mainElement, kind, 'PI);', length: 0),
+      _expectId(mainElement, kind, 'Random()', length: 0),
+      _expectId(mainElement, kind, 'max(', length: 0),
+      _expectId(barElement, kind, 'Random bar()', length: 0),
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ImportElement_withPrefix() async {
+    await _resolveTestUnit('''
+import 'dart:math' as math show max, PI, Random hide min;
+export 'dart:math' show max, PI, Random hide min;
+main() {
+  print(math.PI);
+  print(new math.Random());
+  print(math.max(1, 2));
+}
+math.Random bar() => null;
+''');
+    ImportElement element = testLibraryElement.imports[0];
+    Element mainElement = await _findElement('main');
+    Element barElement = await _findElement('bar');
+    var kind = SearchResultKind.REFERENCE;
+    var length = 'math.'.length;
+    var expected = [
+      _expectId(mainElement, kind, 'math.PI);', length: length),
+      _expectId(mainElement, kind, 'math.Random()', length: length),
+      _expectId(mainElement, kind, 'math.max(', length: length),
+      _expectId(barElement, kind, 'math.Random bar()', length: length),
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ImportElement_withPrefix_forMultipleImports() async {
+    await _resolveTestUnit('''
+import 'dart:async' as p;
+import 'dart:math' as p;
+main() {
+  p.Random;
+  p.Future;
+}
+''');
+    Element mainElement = await _findElement('main');
+    var kind = SearchResultKind.REFERENCE;
+    var length = 'p.'.length;
+    {
+      ImportElement element = testLibraryElement.imports[0];
+      var expected = [
+        _expectId(mainElement, kind, 'p.Future;', length: length),
+      ];
+      await _verifyReferences(element, expected);
+    }
+    {
+      ImportElement element = testLibraryElement.imports[1];
+      var expected = [
+        _expectId(mainElement, kind, 'p.Random', length: length),
+      ];
+      await _verifyReferences(element, expected);
+    }
+  }
+
   test_searchReferences_LabelElement() async {
     await _resolveTestUnit('''
 main() {
@@ -321,6 +583,28 @@ label:
     await _verifyReferences(element, expected);
   }
 
+  test_searchReferences_LibraryElement() async {
+    var codeA = 'part of lib; // A';
+    var codeB = 'part of lib; // B';
+    provider.newFile(_p('$testProject/unitA.dart'), codeA);
+    provider.newFile(_p('$testProject/unitB.dart'), codeB);
+    await _resolveTestUnit('''
+library lib;
+part 'unitA.dart';
+part 'unitB.dart';
+''');
+    LibraryElement element = testLibraryElement;
+    CompilationUnitElement unitElementA = element.parts[0];
+    CompilationUnitElement unitElementB = element.parts[1];
+    var expected = [
+      new ExpectedResult(unitElementA, SearchResultKind.REFERENCE,
+          codeA.indexOf('lib; // A'), 'lib'.length),
+      new ExpectedResult(unitElementB, SearchResultKind.REFERENCE,
+          codeB.indexOf('lib; // B'), 'lib'.length),
+    ];
+    await _verifyReferences(element, expected);
+  }
+
   test_searchReferences_LocalVariableElement() async {
     await _resolveTestUnit(r'''
 main() {
@@ -404,89 +688,7 @@ main(A<int> a) {
     await _verifyReferences(method, expected);
   }
 
-  test_searchReferences_ParameterElement_ofConstructor() async {
-    await _resolveTestUnit('''
-class C {
-  var f;
-  C({p}) : f = p + 1 {
-    p = 2;
-    p += 3;
-    print(p);
-    p();
-  }
-}
-main() {
-  new C(p: 42);
-}
-''');
-    ParameterElement element = _findElement('p');
-    ClassElement classC = _findElement('C');
-    ConstructorElement constructorA = classC.unnamedConstructor;
-    Element mainElement = _findElement('main');
-    var expected = [
-      _expectId(constructorA, SearchResultKind.READ, 'p + 1 {'),
-      _expectId(constructorA, SearchResultKind.WRITE, 'p = 2;'),
-      _expectId(constructorA, SearchResultKind.READ_WRITE, 'p += 3;'),
-      _expectId(constructorA, SearchResultKind.READ, 'p);'),
-      _expectId(constructorA, SearchResultKind.INVOCATION, 'p();'),
-      _expectIdQ(mainElement, SearchResultKind.REFERENCE, 'p: 42')
-    ];
-    await _verifyReferences(element, expected);
-  }
-
-  test_searchReferences_ParameterElement_ofLocalFunction() async {
-    await _resolveTestUnit('''
-main() {
-  foo({p}) {
-    p = 1;
-    p += 2;
-    print(p);
-    p();
-  }
-  foo(p: 42);
-}
-''');
-    ParameterElement element = _findElement('p');
-    Element fooElement = _findElement('foo');
-    Element mainElement = _findElement('main');
-    var expected = [
-      _expectId(fooElement, SearchResultKind.WRITE, 'p = 1;'),
-      _expectId(fooElement, SearchResultKind.READ_WRITE, 'p += 2;'),
-      _expectId(fooElement, SearchResultKind.READ, 'p);'),
-      _expectId(fooElement, SearchResultKind.INVOCATION, 'p();'),
-      _expectIdQ(mainElement, SearchResultKind.REFERENCE, 'p: 42')
-    ];
-    await _verifyReferences(element, expected);
-  }
-
-  test_searchReferences_ParameterElement_ofMethod() async {
-    await _resolveTestUnit('''
-class C {
-  foo({p}) {
-    p = 1;
-    p += 2;
-    print(p);
-    p();
-  }
-}
-main(C c) {
-  c.foo(p: 42);
-}
-''');
-    ParameterElement element = _findElement('p');
-    Element fooElement = _findElement('foo');
-    Element mainElement = _findElement('main');
-    var expected = [
-      _expectId(fooElement, SearchResultKind.WRITE, 'p = 1;'),
-      _expectId(fooElement, SearchResultKind.READ_WRITE, 'p += 2;'),
-      _expectId(fooElement, SearchResultKind.READ, 'p);'),
-      _expectId(fooElement, SearchResultKind.INVOCATION, 'p();'),
-      _expectIdQ(mainElement, SearchResultKind.REFERENCE, 'p: 42')
-    ];
-    await _verifyReferences(element, expected);
-  }
-
-  test_searchReferences_ParameterElement_ofTopLevelFunction() async {
+  test_searchReferences_ParameterElement_named() async {
     await _resolveTestUnit('''
 foo({p}) {
   p = 1;
@@ -511,6 +713,105 @@ main() {
     await _verifyReferences(element, expected);
   }
 
+  test_searchReferences_ParameterElement_ofConstructor() async {
+    await _resolveTestUnit('''
+class C {
+  var f;
+  C(p) : f = p + 1 {
+    p = 2;
+    p += 3;
+    print(p);
+    p();
+  }
+}
+main() {
+  new C(42);
+}
+''');
+    ParameterElement element = _findElement('p');
+    ClassElement classC = _findElement('C');
+    ConstructorElement constructorA = classC.unnamedConstructor;
+    var expected = [
+      _expectId(constructorA, SearchResultKind.READ, 'p + 1 {'),
+      _expectId(constructorA, SearchResultKind.WRITE, 'p = 2;'),
+      _expectId(constructorA, SearchResultKind.READ_WRITE, 'p += 3;'),
+      _expectId(constructorA, SearchResultKind.READ, 'p);'),
+      _expectId(constructorA, SearchResultKind.INVOCATION, 'p();')
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ParameterElement_ofLocalFunction() async {
+    await _resolveTestUnit('''
+main() {
+  foo(p) {
+    p = 1;
+    p += 2;
+    print(p);
+    p();
+  }
+  foo(42);
+}
+''');
+    ParameterElement element = _findElement('p');
+    Element fooElement = _findElement('foo');
+    var expected = [
+      _expectId(fooElement, SearchResultKind.WRITE, 'p = 1;'),
+      _expectId(fooElement, SearchResultKind.READ_WRITE, 'p += 2;'),
+      _expectId(fooElement, SearchResultKind.READ, 'p);'),
+      _expectId(fooElement, SearchResultKind.INVOCATION, 'p();')
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ParameterElement_ofMethod() async {
+    await _resolveTestUnit('''
+class C {
+  foo(p) {
+    p = 1;
+    p += 2;
+    print(p);
+    p();
+  }
+}
+main(C c) {
+  c.foo(42);
+}
+''');
+    ParameterElement element = _findElement('p');
+    Element fooElement = _findElement('foo');
+    var expected = [
+      _expectId(fooElement, SearchResultKind.WRITE, 'p = 1;'),
+      _expectId(fooElement, SearchResultKind.READ_WRITE, 'p += 2;'),
+      _expectId(fooElement, SearchResultKind.READ, 'p);'),
+      _expectId(fooElement, SearchResultKind.INVOCATION, 'p();')
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_ParameterElement_ofTopLevelFunction() async {
+    await _resolveTestUnit('''
+foo(p) {
+  p = 1;
+  p += 2;
+  print(p);
+  p();
+}
+main() {
+  foo(42);
+}
+''');
+    ParameterElement element = _findElement('p');
+    Element fooElement = _findElement('foo');
+    var expected = [
+      _expectId(fooElement, SearchResultKind.WRITE, 'p = 1;'),
+      _expectId(fooElement, SearchResultKind.READ_WRITE, 'p += 2;'),
+      _expectId(fooElement, SearchResultKind.READ, 'p);'),
+      _expectId(fooElement, SearchResultKind.INVOCATION, 'p();')
+    ];
+    await _verifyReferences(element, expected);
+  }
+
   test_searchReferences_PrefixElement() async {
     String partCode = r'''
 part of my_lib;
@@ -539,6 +840,88 @@ main() {
     await _verifyReferences(element, expected);
   }
 
+  test_searchReferences_private_declaredInDefiningUnit() async {
+    String p1 = _p('$testProject/part1.dart');
+    String p2 = _p('$testProject/part2.dart');
+    String p3 = _p('$testProject/part3.dart');
+    String code1 = 'part of lib; _C v1;';
+    String code2 = 'part of lib; _C v2;';
+    provider.newFile(p1, code1);
+    provider.newFile(p2, code2);
+    provider.newFile(p3, 'part of lib; int v3;');
+
+    driver.addFile(p1);
+    driver.addFile(p2);
+    driver.addFile(p3);
+
+    await _resolveTestUnit('''
+library lib;
+part 'part1.dart';
+part 'part2.dart';
+part 'part3.dart';
+class _C {}
+_C v;
+''');
+    ClassElement element = _findElementAtString('_C {}');
+    Element v = testUnitElement.topLevelVariables[0];
+    Element v1 = testLibraryElement.parts[0].topLevelVariables[0];
+    Element v2 = testLibraryElement.parts[1].topLevelVariables[0];
+    var expected = [
+      _expectId(v, SearchResultKind.REFERENCE, '_C v;', length: 2),
+      new ExpectedResult(
+          v1, SearchResultKind.REFERENCE, code1.indexOf('_C v1;'), 2),
+      new ExpectedResult(
+          v2, SearchResultKind.REFERENCE, code2.indexOf('_C v2;'), 2),
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_searchReferences_private_declaredInPart() async {
+    String p = _p('$testProject/lib.dart');
+    String p1 = _p('$testProject/part1.dart');
+    String p2 = _p('$testProject/part2.dart');
+
+    var code = '''
+library lib;
+part 'part1.dart';
+part 'part2.dart';
+_C v;
+''';
+    var code1 = '''
+part of lib;
+class _C {}
+_C v1;
+''';
+    String code2 = 'part of lib; _C v2;';
+
+    provider.newFile(p, code);
+    provider.newFile(p1, code1);
+    provider.newFile(p2, code2);
+
+    driver.addFile(p);
+    driver.addFile(p1);
+    driver.addFile(p2);
+
+    AnalysisResult result = await driver.getResult(p);
+    testUnit = result.unit;
+    testUnitElement = testUnit.element;
+    testLibraryElement = testUnitElement.library;
+
+    ClassElement element = testLibraryElement.parts[0].types[0];
+    Element v = testUnitElement.topLevelVariables[0];
+    Element v1 = testLibraryElement.parts[0].topLevelVariables[0];
+    Element v2 = testLibraryElement.parts[1].topLevelVariables[0];
+    var expected = [
+      new ExpectedResult(
+          v, SearchResultKind.REFERENCE, code.indexOf('_C v;'), 2),
+      new ExpectedResult(
+          v1, SearchResultKind.REFERENCE, code1.indexOf('_C v1;'), 2),
+      new ExpectedResult(
+          v2, SearchResultKind.REFERENCE, code2.indexOf('_C v2;'), 2),
+    ];
+    await _verifyReferences(element, expected);
+  }
+
   test_searchReferences_PropertyAccessorElement_getter() async {
     await _resolveTestUnit('''
 class A {
@@ -682,6 +1065,44 @@ foo<T>(T a) {
     await _verifyReferences(element, expected);
   }
 
+  test_searchSubtypes() async {
+    await _resolveTestUnit('''
+class T {}
+class A extends T {} // A
+class B = Object with T; // B
+class C implements T {} // C
+''');
+    ClassElement element = _findElement('T');
+    ClassElement a = _findElement('A');
+    ClassElement b = _findElement('B');
+    ClassElement c = _findElement('C');
+    var expected = [
+      _expectId(a, SearchResultKind.REFERENCE, 'T {} // A'),
+      _expectId(b, SearchResultKind.REFERENCE, 'T; // B'),
+      _expectId(c, SearchResultKind.REFERENCE, 'T {} // C'),
+    ];
+    await _verifyReferences(element, expected);
+  }
+
+  test_topLevelElements() async {
+    await _resolveTestUnit('''
+class A {} // A
+class B = Object with A;
+typedef C();
+D() {}
+var e = null;
+class NoMatchABCDE {}
+''');
+    Element a = _findElement('A');
+    Element b = _findElement('B');
+    Element c = _findElement('C');
+    Element d = _findElement('D');
+    Element e = _findElement('e');
+    RegExp regExp = new RegExp(r'^[ABCDe]$');
+    expect(await driver.search.topLevelElements(regExp),
+        unorderedEquals([a, b, c, d, e]));
+  }
+
   ExpectedResult _expectId(
       Element enclosingElement, SearchResultKind kind, String search,
       {int length, bool isResolved: true, bool isQualified: false}) {
@@ -702,6 +1123,26 @@ foo<T>(T a) {
     return _expectId(element, kind, search, isQualified: true, length: length);
   }
 
+  /**
+   * Create [ExpectedResult] for a qualified and unresolved match.
+   */
+  ExpectedResult _expectIdQU(
+      Element element, SearchResultKind kind, String search,
+      {int length}) {
+    return _expectId(element, kind, search,
+        isQualified: true, isResolved: false, length: length);
+  }
+
+  /**
+   * Create [ExpectedResult] for a unqualified and unresolved match.
+   */
+  ExpectedResult _expectIdU(
+      Element element, SearchResultKind kind, String search,
+      {int length}) {
+    return _expectId(element, kind, search,
+        isQualified: false, isResolved: false, length: length);
+  }
+
   Element _findElement(String name, [ElementKind kind]) {
     return findChildElement(testUnit.element, name, kind);
   }
@@ -724,6 +1165,14 @@ foo<T>(T a) {
     }
   }
 
+  Future<Null> _verifyNameReferences(
+      String name, List<ExpectedResult> expectedMatches) async {
+    List<SearchResult> results =
+        await driver.search.unresolvedMemberReferences(name);
+    _assertResults(results, expectedMatches);
+    expect(results, hasLength(expectedMatches.length));
+  }
+
   Future _verifyReferences(
       Element element, List<ExpectedResult> expectedMatches) async {
     List<SearchResult> results = await driver.search.references(element);
